-- PC6001 top module
--

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity RGBCONV is
	port (
		LEDGN			: out   std_logic_vector(2 downto 0);	-- LED
		PUSHSWN			: in    std_logic;						-- push sw
		PWRONRST		: in    std_logic;						-- power on reset
		CLK50MIN		: in    std_logic;						-- clock input 50M osc
		CLKIN			: in    std_logic_vector( 6 downto 0);	-- clock input #2
		GPIO0_D			: inout std_logic_vector( 9 downto 0);	-- GPIO data
		GPIO1_D			: inout std_logic_vector(23 downto 0);	-- GPIO data
		GPIO2_D			: inout std_logic_vector(17 downto 0);	-- GPIO data
		GPIO3_D			: inout std_logic_vector(21 downto 0) 	-- GPIO data
	);
end RGBCONV;

-- P4i`bvj
--  1:GND                   02:GND                 
--  3:LEDGN[0]    (pin  3)  04:GPIO0_D[ 0] (pin  4)
--  5:LEDGN[1]    (pin  7)  06:GPIO0_D[ 1] (pin  8)
--  7:LEDGN[2]    (pin  9)  08:GND                 
--  9:GND                   10:GND                 
-- 11:CLKIN[0]    (pin 18)  12:CLK50MIN    (pin 17)
-- 13:CLKIN[2]    (pin 22)  14:CLKIN[1]    (pin 21)
-- 15:GPIO0_D[ 2] (pin 25)  16:GPIO0_D[ 3] (pin 24)
-- 17:GPIO0_D[ 4] (pin 27)  18:GPIO0_D[ 5] (pin 26)
-- 19:VCC1.2V               20:VCC1.2V             
-- 21:GPIO0_D[ 6] (pin 30)  22:GPIO0_D[ 7] (pin 28)
-- 23:GPIO0_D[ 8] (pin 32)  24:GPIO0_D[ 9] (pin 31)
-- 25:GND                   26:GND                 
-- 27:VCC3.3V               28:VCC3.3V             

-- P1i`bvj
--  1:GPIO1_D[ 0] (pin 40)  02:GPIO1_D[ 1] (pin 41)
--  3:GPIO1_D[ 2] (pin 42)  04:GPIO1_D[ 3] (pin 43)
--  5:GPIO1_D[ 4] (pin 44)  06:GPIO1_D[ 5] (pin 45)
--  7:GPIO1_D[ 6] (pin 47)  08:GPIO1_D[ 7] (pin 48)
--  9:GPIO1_D[ 8] (pin 51)  10:GPIO1_D[ 9] (pin 52)
-- 11:GPIO1_D[10] (pin 53)  12:GPIO1_D[11] (pin 55)
-- 13:GPIO1_D[12] (pin 57)  14:GPIO1_D[13] (pin 58)
-- 15:GPIO1_D[14] (pin 59)  16:GPIO1_D[15] (pin 60)
-- 17:GPIO1_D[16] (pin 63)  18:GPIO1_D[17] (pin 64)
-- 19:GPIO1_D[18] (pin 65)  20:GPIO1_D[19] (pin 67)
-- 21:GPIO1_D[20] (pin 69)  22:GPIO1_D[21] (pin 70)
-- 23:GPIO1_D[22] (pin 71)  24:GPIO1_D[23] (pin 72)
-- 25:GND                   26:GND                 
-- 27:VCC3.3V               28:VCC3.3V             

-- P2i`bvEj
--  1:GND                   02:GND                 
--  3:PWRONRST    (pin 73)  04:GPIO2_D[ 0] (pin 74)
--  5:GPIO2_D[ 1] (pin 75)  06:GPIO2_D[ 2] (pin 76)
--  7:GPIO2_D[ 3] (pin 79)  08:GPIO2_D[ 4] (pin 80)
--  9:GPIO2_D[ 5] (pin 81)  10:GPIO2_D[ 6] (pin 86)
-- 11:GPIO2_D[ 7] (pin 87)  12:CLKIN[3]    (pin 88)
-- 13:CLKIN[4]    (pin 89)  14:CLKIN[5]    (pin 90)
-- 15:CLKIN[6]    (pin 91)  16:GPIO2_D[ 8] (pin 92)
-- 17:GPIO2_D[ 9] (pin 93)  18:GPIO2_D[10] (pin 94)
-- 19:GPIO2_D[11] (pin 96)  20:GPIO2_D[12] (pin 97)
-- 21:GPIO2_D[13] (pin 99)  22:GPIO2_D[14] (pin100)
-- 23:GPIO2_D[15] (pin101)  24:GPIO2_D[16] (pin103)
-- 25:GPIO2_D[17] (pin104)  26:GND                 
-- 27:GND                   28:VCC3.3V             

-- P3i`bvj
--  1:GND                   02:GND                 
--  3:GPIO3_D[ 0] (pin112)  04:GPIO3_D[ 1] (pin113)
--  5:GPIO3_D[ 2] (pin114)  06:GPIO3_D[ 3] (pin115)
--  7:GPIO3_D[ 4] (pin118)  08:GPIO3_D[ 5] (pin119)
--  9:GPIO3_D[ 6] (pin120)  10:GPIO3_D[ 7] (pin121)
-- 11:GPIO3_D[ 8] (pin122)  12:GPIO3_D[ 9] (pin125)
-- 13:GPIO3_D[10] (pin126)  14:GPIO3_D[11] (pin129)
-- 15:GPIO3_D[12] (pin132)  16:GPIO3_D[13] (pin133)
-- 17:GPIO3_D[14] (pin134)  18:GPIO3_D[15] (pin135)
-- 19:GPIO3_D[16] (pin136)  20:GPIO3_D[17] (pin137)
-- 21:GPIO3_D[18] (pin139)  22:GPIO3_D[19] (pin141)
-- 23:GPIO3_D[20] (pin142)  24:GPIO3_D[21] (pin143)
-- 25:PUSHSWN     (pin144)  26:GND                 
-- 27:GND                   28:VCC3.3V             


architecture RTL of RGBCONV is

	component CLKGEN is
		port (
			CLK50M		: in  std_logic;
			RSTN		: in  std_logic;
			CLK100MOUT	: out std_logic;
			LOCK_PLL	: out std_logic
		);
	end component;

	component P6SYNC is
		port (
			P6_HSYNCN	: in  std_logic;
			P6_VSYNCN	: in  std_logic;
			P6_RGBH		: in  std_logic_vector(3 downto 0);
			P6_CLK		: in  std_logic;
			CLK100M		: in  std_logic;
			RSTN		: in  std_logic;
			SRMODE		: out std_logic;
			CLKENB		: out std_logic;
			DT_HSYNCN	: out std_logic;
			DT_VSYNCN	: out std_logic;
			DT_RGBH		: out std_logic_vector(3 downto 0);
			SFTDT_U		: out std_logic_vector(2 downto 0);
			SFTDT_D		: out std_logic_vector(2 downto 0);
			SFTHS_U		: out std_logic_vector(2 downto 0);
			SFTHS_D		: out std_logic_vector(2 downto 0);
			LOCK1		: out std_logic;
			LOCK2		: out std_logic
		);
	end component;

	component DATASFT is
		port (
			HSYNCN		: in  std_logic;
			VSYNCN		: in  std_logic;
			RGBH		: in  std_logic_vector(3 downto 0);
			CLKENB		: in  std_logic;
			SRMODE		: in  std_logic;
			LOCK1		: in  std_logic;
			LOCK2		: in  std_logic;
			CLK100M		: in  std_logic;
			RSTN		: in  std_logic;
			HSYNCN_O	: out std_logic;
			VSYNCN_O	: out std_logic;
			RGBH_O		: out std_logic_vector(3 downto 0);
			LOCK3		: out std_logic;
			LOCK4		: out std_logic
		);
	end component;

	component VGAOUT is
		port (
			DISPD		: in  std_logic_vector(3 downto 0);
			DISPTMG_HS	: in  std_logic;
			DISPTMG_VS	: in  std_logic;
			CLKENB		: in  std_logic;
			SRMODE		: in  std_logic;
			SC4COLORON	: in  std_logic;
			SC4COLORMD	: in  std_logic_vector(1 downto 0);
			RAMDTI		: in  std_logic_vector(15 downto 0);
			DT_LOCK3	: in  std_logic;
			DT_LOCK4	: in  std_logic;
			LOCK_PLL	: in  std_logic;
			CLK100M		: in  std_logic;
			RSTN		: in  std_logic;
			RAMADD		: out std_logic_vector(14 downto 0);
			RAMDTO		: out std_logic_vector(15 downto 0);
			RAMWRN		: out std_logic;
			RAMOEN		: out std_logic;
			RAMCTRLN	: out std_logic;
			VGA_R		: out std_logic_vector(3 downto 0);
			VGA_G		: out std_logic_vector(3 downto 0);
			VGA_B		: out std_logic_vector(3 downto 0);
			VGA_HS		: out std_logic;
			VGA_VS		: out std_logic
		);
	end component;

	component COLSEL is
		port (
			MDSEL		: in  std_logic;
			LOCK_PLL	: in  std_logic;
			CLK100M		: in  std_logic;
			RSTN		: in  std_logic;
			COLORMD		: out std_logic_vector(2 downto 0)
		);
	end component;

-- PC-6001 input
	signal p6_r			: std_logic;
	signal p6_g			: std_logic;
	signal p6_b			: std_logic;
	signal p6_h			: std_logic;
	signal p6_hsyncn	: std_logic;
	signal p6_vsyncn	: std_logic;
	signal p6_rgbh		: std_logic_vector(3 downto 0);
	signal p6_clk		: std_logic;

-- clkgen
	signal rstn			: std_logic;
	signal clk100m		: std_logic;
	signal lock_pll		: std_logic;

-- p6sync
	signal sy_hsyncn	: std_logic;
	signal sy_vsyncn	: std_logic;
	signal sy_srmode	: std_logic;
	signal sy_clkenb	: std_logic;
	signal sy_rgbh		: std_logic_vector(3 downto 0);
	signal sy_sftdt_u	: std_logic_vector(2 downto 0);
	signal sy_sftdt_d	: std_logic_vector(2 downto 0);
	signal sy_sfths_u	: std_logic_vector(2 downto 0);
	signal sy_sfths_d	: std_logic_vector(2 downto 0);
	signal sy_lock1		: std_logic;
	signal sy_lock2		: std_logic;

-- datasft
	signal dt_hsyncn	: std_logic;
	signal dt_vsyncn	: std_logic;
	signal dt_rgbh		: std_logic_vector(3 downto 0);
	signal dt_lock3		: std_logic;
	signal dt_lock4		: std_logic;

-- vgaout
	signal vga_r_i		: std_logic_vector(3 downto 0);
	signal vga_g_i		: std_logic_vector(3 downto 0);
	signal vga_b_i		: std_logic_vector(3 downto 0);
	signal vga_hs_i		: std_logic;
	signal vga_vs_i		: std_logic;
	signal ram_add		: std_logic_vector(14 downto 0);
	signal ram_dti		: std_logic_vector(15 downto 0);
	signal ram_dto		: std_logic_vector(15 downto 0);
	signal ram_wrn		: std_logic;
	signal ram_oen		: std_logic;
	signal ram_ctrln	: std_logic;

-- colormd
	signal mdsel		: std_logic;
	signal colormd		: std_logic_vector(2 downto 0);



-- prob
	signal probdat1		: std_logic_vector(7 downto 0);
	signal probdat2		: std_logic_vector(15 downto 0);
	signal probdat3		: std_logic_vector(7 downto 0);

	signal gpio0_d_tmp		: std_logic_vector(31 downto 0);
	signal gpio0_clkout_tmp	: std_logic_vector(1 downto 0);

	signal gpio1_d_tmp		: std_logic_vector(31 downto 0);
	signal gpio1_clkout_tmp	: std_logic_vector(1 downto 0);

begin

-- clock generate
	rstn <= not PWRONRST;

	U_CLKGEN : CLKGEN
	port map (
		CLK50M		=> CLK50MIN,
		RSTN		=> rstn,
		CLK100MOUT	=> clk100m,
		LOCK_PLL	=> lock_pll
	);


-- clock detect and data latch

	p6_rgbh <= p6_h & p6_g & p6_r & p6_b;

	U_P6SYNC : P6SYNC
	port map (
		P6_HSYNCN	=> p6_hsyncn,
		P6_VSYNCN	=> p6_vsyncn,
		P6_RGBH		=> p6_rgbh,
		P6_CLK		=> p6_clk,
		CLK100M		=> clk100m,
		RSTN		=> rstn,
		SRMODE		=> sy_srmode,
		CLKENB		=> sy_clkenb,
		DT_HSYNCN	=> sy_hsyncn,
		DT_VSYNCN	=> sy_vsyncn,
		DT_RGBH		=> sy_rgbh,
		SFTDT_U		=> open,
		SFTDT_D		=> open,
		SFTHS_U		=> open,
		SFTHS_D		=> open,
		LOCK1		=> sy_lock1,
		LOCK2		=> sy_lock2
	);

-- data shift
	U_DATASFT : DATASFT
	port map (
		HSYNCN		=> sy_hsyncn,
		VSYNCN		=> sy_vsyncn,
		RGBH		=> sy_rgbh,
		CLKENB		=> sy_clkenb,
		SRMODE		=> sy_srmode,
		LOCK1		=> sy_lock1,
		LOCK2		=> sy_lock2,
		CLK100M		=> clk100m,
		RSTN		=> rstn,
		HSYNCN_O	=> dt_hsyncn,
		VSYNCN_O	=> dt_vsyncn,
		RGBH_O		=> dt_rgbh,
		LOCK3		=> dt_lock3,
		LOCK4		=> dt_lock4
	);


-- VGA output
	U_VGAOUT : VGAOUT
	port map (
		DISPD		=> dt_rgbh,
		DISPTMG_HS	=> dt_hsyncn,
		DISPTMG_VS	=> dt_vsyncn,
		CLKENB		=> sy_clkenb,
		SRMODE		=> sy_srmode,
		SC4COLORON	=> colormd(2),
		SC4COLORMD	=> colormd(1 downto 0),
		RAMDTI		=> ram_dti,
		DT_LOCK3	=> dt_lock3,
		DT_LOCK4	=> dt_lock4,
		LOCK_PLL	=> lock_pll,
		CLK100M		=> clk100m,
		RSTN		=> rstn,
		RAMADD		=> ram_add,
		RAMDTO		=> ram_dto,
		RAMWRN		=> ram_wrn,
		RAMOEN		=> ram_oen,
		RAMCTRLN	=> ram_ctrln,
		VGA_R		=> vga_r_i,
		VGA_G		=> vga_g_i,
		VGA_B		=> vga_b_i,
		VGA_HS		=> vga_hs_i,
		VGA_VS		=> vga_vs_i
	);

	GPIO1_D(23) <= vga_r_i(0);
	GPIO1_D(21) <= vga_r_i(1);
	GPIO1_D(19) <= vga_r_i(2);
	GPIO1_D(17) <= vga_r_i(3);

	GPIO1_D(15) <= vga_g_i(0);
	GPIO1_D(13) <= vga_g_i(1);
	GPIO1_D(11) <= vga_g_i(2);
	GPIO1_D(9)  <= vga_g_i(3);

	GPIO1_D(7)  <= vga_b_i(0);
	GPIO1_D(5)  <= vga_b_i(1);
	GPIO1_D(3)  <= vga_b_i(2);
	GPIO1_D(1)  <= vga_b_i(3);

	GPIO3_D(21) <= vga_vs_i;
	GPIO3_D(19) <= vga_hs_i;

	GPIO1_D(18) <= ram_oen;
	GPIO3_D(16) <= ram_wrn;

	GPIO1_D( 0) <= ram_add(0);
	GPIO1_D( 2) <= ram_add(1);
	GPIO1_D( 4) <= ram_add(2);
	GPIO1_D( 6) <= ram_add(3);
	GPIO1_D( 8) <= ram_add(4);
	GPIO1_D(20) <= ram_add(5);
	GPIO1_D(22) <= ram_add(6);
	GPIO3_D( 0) <= ram_add(7);
	GPIO3_D( 2) <= ram_add(8);
	GPIO3_D( 4) <= ram_add(9);
	GPIO3_D( 6) <= ram_add(10);
	GPIO3_D(15) <= ram_add(11);
	GPIO3_D(17) <= ram_add(12);
	GPIO3_D(18) <= ram_add(13);
	GPIO3_D(20) <= ram_add(14);

	GPIO1_D(10) <= ram_dto( 0) when (ram_ctrln='0') else 'Z';
	GPIO1_D(12) <= ram_dto( 1) when (ram_ctrln='0') else 'Z';
	GPIO1_D(14) <= ram_dto( 2) when (ram_ctrln='0') else 'Z';
	GPIO1_D(16) <= ram_dto( 3) when (ram_ctrln='0') else 'Z';
	GPIO3_D( 8) <= ram_dto( 4) when (ram_ctrln='0') else 'Z';
	GPIO3_D(10) <= ram_dto( 5) when (ram_ctrln='0') else 'Z';
	GPIO3_D(12) <= ram_dto( 6) when (ram_ctrln='0') else 'Z';
	GPIO3_D(14) <= ram_dto( 7) when (ram_ctrln='0') else 'Z';
	GPIO0_D( 2) <= ram_dto( 8) when (ram_ctrln='0') else 'Z';
	GPIO0_D( 3) <= ram_dto( 9) when (ram_ctrln='0') else 'Z';
	GPIO0_D( 4) <= ram_dto(10) when (ram_ctrln='0') else 'Z';
	GPIO0_D( 5) <= ram_dto(11) when (ram_ctrln='0') else 'Z';
	GPIO0_D( 6) <= ram_dto(12) when (ram_ctrln='0') else 'Z';
	GPIO0_D( 7) <= ram_dto(13) when (ram_ctrln='0') else 'Z';
	GPIO0_D( 8) <= ram_dto(14) when (ram_ctrln='0') else 'Z';
	GPIO0_D( 9) <= ram_dto(15) when (ram_ctrln='0') else 'Z';


	ram_dti( 0)  <= GPIO1_D(10);
	ram_dti( 1)  <= GPIO1_D(12);
	ram_dti( 2)  <= GPIO1_D(14);
	ram_dti( 3)  <= GPIO1_D(16);
	ram_dti( 4)  <= GPIO3_D( 8);
	ram_dti( 5)  <= GPIO3_D(10);
	ram_dti( 6)  <= GPIO3_D(12);
	ram_dti( 7)  <= GPIO3_D(14);
	ram_dti( 8)  <= GPIO0_D( 2);
	ram_dti( 9)  <= GPIO0_D( 3);
	ram_dti(10)  <= GPIO0_D( 4);
	ram_dti(11)  <= GPIO0_D( 5);
	ram_dti(12)  <= GPIO0_D( 6);
	ram_dti(13)  <= GPIO0_D( 7);
	ram_dti(14)  <= GPIO0_D( 8);
	ram_dti(15)  <= GPIO0_D( 9);

-- color mode select

	mdsel <= not PUSHSWN;

	U_COLSEL : COLSEL
	port map (
		MDSEL		=> mdsel,
		LOCK_PLL	=> lock_pll,
		CLK100M		=> clk100m,
		RSTN		=> rstn,
		COLORMD		=> colormd
	);


	p6_h      <= GPIO3_D(1);
	p6_b      <= GPIO3_D(3);
	p6_g      <= GPIO3_D(5);
	p6_r      <= GPIO3_D(7);
	p6_hsyncn <= GPIO3_D(9);
	p6_vsyncn <= GPIO3_D(11);
	p6_clk    <= GPIO3_D(13);


	LEDGN(0) <= not colormd(2);
	LEDGN(1) <= not sy_srmode;
	LEDGN(2) <= not (dt_lock3 or dt_lock4);


-- unused pin

	GPIO0_D(1 downto 0) <= (others => 'Z');
	GPIO2_D <= (others => 'Z');


end RTL;
